#!/usr/bin/env python
from __future__ import (absolute_import, division, print_function,
                            unicode_literals)
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="genereate .eps file",action="store_true")
args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')

import numpy as np
import matplotlib.pyplot as plt
import ellc

# Note that star 1 is the star labelled Secondary in nightfall
phase = np.linspace(-0.3,0.8,num=1001)
t_zero = 0.0
period = 4.0
sbratio = 1.4
r_1 = 0.09992 
r_2 = 0.09726
incl = 89.99
a = 16.826
q = 1/0.8
rotfac_1 = 1
rotfac_2 = 15
shape_1 = 'roche'
shape_2 = 'roche'
gdc_1 = 1
gdc_2 = 1
ecc = 0  # e>0 does not work in nightfall!
om  = 90
f_c = np.sqrt(ecc)*np.cos(om*np.pi/180.)
f_s = np.sqrt(ecc)*np.sin(om*np.pi/180.)

t = phase*period

ld_1 = 'claret'
ldc_1 = [0.6, 0, 0, 0] # 
ld_2 = 'claret'
ldc_2 = [0, 0, 0.3, 0] # 

lc = ellc.lc(t,t_zero=t_zero, period=period, a=a, q=q,
    ld_1=ld_1,ldc_1=ldc_1,ld_2=ld_2,ldc_2=ldc_2,
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio, 
    rotfac_1=rotfac_1, rotfac_2=rotfac_2, gdc_1=gdc_1, gdc_2=gdc_2,
    shape_1=shape_1,shape_2=shape_2, grid_1='default',grid_2='default') 
rv_2,rv_1 = ellc.rv(t,t_zero=t_zero, period=period, a=a, q=q,
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio, 
    rotfac_1=rotfac_1, rotfac_2=rotfac_2, gdc_1=gdc_1, gdc_2=gdc_2,
    shape_1=shape_1,shape_2=shape_2, grid_1='default',grid_2='default') 

ld_1 = 'power-2'
ldc_1 = [0.6, 0.5] # 
ld_2 = 'power-2'
ldc_2 = [0.3, 1.5] # 
lc_2 = ellc.lc(t,t_zero=t_zero, period=period, a=a, q=q,
    ld_1=ld_1,ldc_1=ldc_1,ld_2=ld_2,ldc_2=ldc_2,
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio, 
    rotfac_1=rotfac_1, rotfac_2=rotfac_2, gdc_1=gdc_1, gdc_2=gdc_2,
    shape_1=shape_1,shape_2=shape_2, grid_1='default',grid_2='default') 
rv_2_2,rv_2_1 = ellc.rv(t,t_zero=t_zero, period=period, a=a, q=q,
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio, 
    rotfac_1=rotfac_1, rotfac_2=rotfac_2, gdc_1=gdc_1, gdc_2=gdc_2,
    shape_1=shape_1,shape_2=shape_2, grid_1='default',grid_2='default') 

print('Maximum absolute deviation for light curve = {}'.format(
    max(abs(lc-lc_2))))
print('Maximum absolute deviation for rv curve 1 = {}'.format(
    max(abs(rv_1-rv_2_1))))
print('Maximum absolute deviation for rv curve 2 = {}'.format(
    max(abs(rv_2-rv_2_2))))

fontsize=9
fig=plt.figure(1,figsize=(8,4))
fig=plt.figure(1)
plt.subplot(211)
plt.xlim([-0.25,0.75])
plt.ylim([0.4,1.1])
plt.plot(phase,lc,color='darkblue',linestyle='--')
plt.plot(phase,lc_2,color='darkgreen',linestyle=':')
plt.xlabel("Time [d]",fontsize=fontsize)
plt.ylabel("Flux",fontsize=fontsize)
plt.tick_params(axis='both', labelsize=fontsize)

plt.subplot(223)
plt.xlim([-0.05,0.05])
plt.ylim([-50,50])
plt.plot(phase,rv_1,color='darkblue',linestyle='--' )
plt.plot(phase,rv_2,color='darkgreen',linestyle='--')
plt.plot(phase,rv_2_1,color='darkblue',linestyle=':' )
plt.plot(phase,rv_2_2,color='darkgreen',linestyle=':')
plt.locator_params(axis = 'x', nbins = 4)
plt.tick_params(axis='both', labelsize=fontsize)
plt.xlabel("Time [d]",fontsize=fontsize)
plt.ylabel("Radial velocity [km/s]",fontsize=fontsize)
plt.tick_params(axis='both', labelsize=fontsize)

plt.subplot(224)
plt.xlim([0.45,0.55])
plt.ylim([-400,400])
plt.plot(phase,rv_1,color='darkblue',linestyle='--' )
plt.plot(phase,rv_2,color='darkgreen',linestyle='--')
plt.plot(phase,rv_2_1,color='darkblue',linestyle=':' )
plt.plot(phase,rv_2_2,color='darkgreen',linestyle=':')
plt.locator_params(axis = 'x', nbins = 4)
plt.tick_params(axis='both', labelsize=fontsize)
plt.ylabel("Radial velocity [km/s]",fontsize=fontsize)
plt.xlabel("Time [d]",fontsize=fontsize)

plt.tight_layout()
if args.eps:
  fig.savefig("power-2.eps")
else:
  plt.show()




""" 
 Return limb-darkening and gravity-darkening coefficients

  The coefficients for a 4-parameter limb-darkening law and the gravity
 darkening coefficient, y, are interpolated from the tables provided by Claret
 and Bloemen (2011A&A...529A..75C) for ATLAS stellar atmosphere models and an
 assumed micro-turbulent velocity xi=2 km/s calculated using a least-squares
 fit. Also available are limb-darkening coefficients for the SDSS u', g', r',
 i' and z' bands from Claret (2004A&A...428.1001C), but no gravity darkening
 coefficient is available for these bands.


  To interpolate the limb-darkening and gravity-darkening coefficients at a
  given effective temperature (T_eff in K), surface gravity (log(g) in cgs
  units) and metallicity ([M/H] logarithmic metal abundance relative to
  solar), create an instance of the class LimbGravityDarkeningCoeffs for the
  desired photometric band and then call this instance with the parameters
  (T_eff, log(g), [M/H]). The first 4 values in the array returned are the
  limb-darkening coefficients a_1 .. a_4 and the final value is the
  gravity-darkening coefficient, y.  If no gravity-darkening coefficient is
  available then the fifth returned value will be None. 

  If the input values of T_eff, log(g) or [M/H] are outside the tabulated
  range then all coefficients are returned as NaN.

   To see the names of the available photometric bands use the function
   ellc.ldy.list_bands(). For the SDSS bands, 'u_' = u', 'g_' = g', etc.

 Example
 -------
  
 >>> from ellc.ldy import LimbGravityDarkeningCoeffs, list_bands
 >>> print(list_bands())
 ['B', 'C', 'H', 'I', 'J', 'K', 'Kp', 'R', 'S1', 'S2', 'S3', 'S4', 'U', 'V',
 'b', 'u', 'v', 'y', 'u_', 'g_', 'r_', 'i_', 'z_']
 >>> ldy_Kp = LimbGravityDarkeningCoeffs('Kp')
 >>> Teff = 10450
 >>> logg = 3.9
 >>> M_H = -0.31
 >>> a1,a2,a3,a4,y = ldy_Kp(Teff, logg, M_H)

"""
  
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
import numpy as np
from astropy.io import fits
from os.path import join,abspath,dirname
from scipy.interpolate import RegularGridInterpolator

dir_path = dirname(abspath(__file__))
ld_file_1 = join(dir_path,'data','g_Claret2011.fits')
ld_file_2 = join(dir_path,'data','g_Claret2004.fits')

def list_bands():
  hdulist = fits.open(ld_file_1)
  list_1 = (hdulist['band'].data)['BAND']
  hdulist.close()

  hdulist = fits.open(ld_file_2)
  list_2 = (hdulist['band'].data)['BAND']
  hdulist.close()

  return  [x.strip(' ') for x in  np.append(list_1, list_2)]

class LimbGravityDarkeningCoeffs:

  def __init__(self, band):
    self._band = band
    if band in ('u_','g_','r_','i_','z_'):
      hdulist = fits.open(ld_file_2)
      teff = hdulist['Teff'].data
    else:
      hdulist = fits.open(ld_file_1)
      logteff = hdulist['logTeff'].data
    logg = hdulist['logg'].data
    M_H = hdulist['M_H'].data
    bands = hdulist['band'].data
    i_band = (bands['BAND'].rfind(band) > -1).nonzero()[0][0]
    grid = (hdulist[0].data)[i_band,:,:,:,:]
    hdulist.close()
    if band in ('u_','g_','r_','i_','z_') :
      pts = (M_H,logg,teff)
      self._a1 = RegularGridInterpolator(pts,grid[0,:,:,:],fill_value=None)
      self._a2 = RegularGridInterpolator(pts,grid[1,:,:,:],fill_value=None)
      self._a3 = RegularGridInterpolator(pts,grid[2,:,:,:],fill_value=None)
      self._a4 = RegularGridInterpolator(pts,grid[3,:,:,:],fill_value=None)
    else:
      pts = (M_H,logg,logteff)
      self._y  = RegularGridInterpolator(pts,grid[0,:,:,:],fill_value=None)
      self._a1 = RegularGridInterpolator(pts,grid[1,:,:,:],fill_value=None)
      self._a2 = RegularGridInterpolator(pts,grid[2,:,:,:],fill_value=None)
      self._a3 = RegularGridInterpolator(pts,grid[3,:,:,:],fill_value=None)
      self._a4 = RegularGridInterpolator(pts,grid[4,:,:,:],fill_value=None)

  def __call__(self, teff, logg, M_H):
    if teff < 0:
      y,a1,a2,a3,a4  = [np.nan,np.nan,np.nan,np.nan,np.nan]
    else:
      if self._band in ('u_','g_','r_','i_','z_'):
        try: 
          y  = None
          a1 = self._a1([M_H,logg,teff])
          a2 = self._a2([M_H,logg,teff])
          a3 = self._a3([M_H,logg,teff])
          a4 = self._a4([M_H,logg,teff])
        except:
          y,a1,a2,a3,a4  = [np.nan,np.nan,np.nan,np.nan,np.nan]
      else:
        try:
          y  = self._y ([M_H,logg,np.log10(teff)])
          a1 = self._a1([M_H,logg,np.log10(teff)])
          a2 = self._a2([M_H,logg,np.log10(teff)])
          a3 = self._a3([M_H,logg,np.log10(teff)])
          a4 = self._a4([M_H,logg,np.log10(teff)])
        except:
          y,a1,a2,a3,a4  = [np.nan,np.nan,np.nan,np.nan,np.nan]

    return (np.array([a1, a2, a3, a4, y])).flatten()
